from django.db import models
from django.utils.translation import ugettext_lazy as _

from picklefield.fields import PickledObjectField


class Pickle(models.Model):
    TYPE_STR = 'str'
    TYPE_INT = 'int'
    TYPE_FLOAT = 'float'
    TYPE_LIST = 'list'
    TYPE_DICT = 'dict'
    TYPE_BOOL = 'bool'
    TYPE_CHOICES = [
        (TYPE_STR, 'STR'),
        (TYPE_INT, 'INT'),
        (TYPE_FLOAT, 'FLOAT'),
        (TYPE_LIST, 'LIST'),
        (TYPE_DICT, 'DICT'),
        (TYPE_BOOL, 'BOOL'),

    ]
    name = models.CharField(verbose_name=_('name'), max_length=100, unique=True)
    value = PickledObjectField(verbose_name=_('value'))
    data_type = models.CharField(verbose_name=_('value data type'), choices=TYPE_CHOICES, max_length=10)

    @property
    def data_value(self):
        return self.value



# Generated by Django 3.1 on 2022-01-21 09:41

from django.db import migrations, models
import picklefield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Pickle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='name')),
                ('value', picklefield.fields.PickledObjectField(editable=False, verbose_name='value')),
                ('data_type', models.CharField(choices=[('str', 'STR'), ('int', 'INT'), ('float', 'FLOAT'), ('list', 'LIST'), ('dict', 'DICT'), ('bool', 'BOOL')], max_length=10, verbose_name='value data type')),
            ],
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from django import forms
from django.core.exceptions import ValidationError

from .models import Pickle


class ConfigAdminForm(forms.ModelForm):
    raw_value = forms.CharField(widget=forms.Textarea())

    class Meta:
        model = Pickle
        fields = ['name', 'raw_value', 'data_type']

    def clean(self):
        raw_value = self.cleaned_data.get('raw_value')
        data_type = self.cleaned_data.get('data_type')
        try:
            data = eval(f"{data_type}({raw_value})")
            self.cleaned_data.update({"raw_value": data})
        except Exception:
            raise ValidationError(_('Enter your raw value in the shape of entered data type'))



from django.contrib import admin

from .models import Pickle
from .forms import ConfigAdminForm


@admin.register(Pickle)
class ConfigModelAdmin(admin.ModelAdmin):
    form = ConfigAdminForm
    list_display = ('name', 'data_type', 'data_value')
    search_fields = ('name',)
    list_filter = ('data_type',)

    def get_form(self, request, obj=None, change=False, **kwargs):
        form = super(ConfigModelAdmin, self).get_form(request, obj, change, **kwargs)
        if obj is not None:
            form.base_fields['raw_value'].initial = obj.value
        return form

    def save_model(self, request, obj, form, change):
        raw_value = form.cleaned_data.get('raw_value')
        obj.value = raw_value
        return super(ConfigModelAdmin, self).save_model(request, obj, form, change)



from rest_framework import views
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework_simplejwt.authentication import JWTAuthentication

from ..models import Pickle


class PicklesAPIView(views.APIView):
    """Shows init configs params"""
    authentication_classes = (JWTAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, *args, **kwargs):
        configs = Pickle.objects.all()
        data = {}
        if configs:
            for config in configs:
                data.update({config.name: config.value})
        return Response(data)
